package core

import (
	"git.urbach.dev/cli/q/src/ast"
	"git.urbach.dev/cli/q/src/errors"
	"git.urbach.dev/cli/q/src/token"
)

// CompileAssign compiles an assign statement.
func (f *Function) CompileAssign(node *ast.Assign) error {
	left := node.Expression.Children[0]
	right := node.Expression.Children[1]

	if left.IsLeaf() {
		name := left.Token.Text(f.File.Bytes)
		variable := f.VariableByName(name)

		if variable == nil {
			return errors.New(&errors.UnknownIdentifier{Name: name}, f.File, left.Token.Position)
		}

		defer f.UseVariable(variable)
		return f.Execute(node.Expression.Token, variable.Register, right)
	}

	if left.Token.Kind == token.Dot {
		return f.CompileAssignField(node)
	}

	if left.Token.Kind == token.Array {
		return f.CompileAssignArray(node)
	}

	if left.Token.Kind == token.Separator && right.Token.Kind == token.Div {
		return f.CompileAssignDivision(node.Expression)
	}

	if !ast.IsFunctionCall(right) {
		return errors.New(errors.NotImplemented, f.File, node.Expression.Token.Position)
	}

	return f.MultiAssign(left, right)
}
