package core

import (
	"math"

	"git.urbach.dev/cli/q/src/asm"
	"git.urbach.dev/cli/q/src/ast"
	"git.urbach.dev/cli/q/src/errors"
	"git.urbach.dev/cli/q/src/types"
)

// CompileAssignField compiles a memory write to a struct field.
func (f *Function) CompileAssignField(node *ast.Assign) error {
	destination := node.Expression.Children[0]
	value := node.Expression.Children[1]
	name := destination.Children[0].Token.Text(f.File.Bytes)
	fieldName := destination.Children[1].Token.Text(f.File.Bytes)
	variable := f.VariableByName(name)

	if variable == nil {
		return errors.New(&errors.UnknownIdentifier{Name: name}, f.File, destination.Children[0].Token.Position)
	}

	defer f.UseVariable(variable)
	pointer := variable.Type.(*types.Pointer)
	structure := pointer.To.(*types.Struct)
	field := structure.FieldByName(fieldName)

	if field == nil {
		return errors.New(&errors.UnknownStructField{StructName: structure.Name(), FieldName: fieldName}, f.File, destination.Children[1].Token.Position)
	}

	memory := asm.Memory{
		Base:           variable.Register,
		Offset:         int8(field.Offset),
		OffsetRegister: math.MaxUint8,
		Length:         byte(field.Type.Size()),
	}

	_, err := f.ExpressionToMemory(value, memory)
	return err
}
