package core

import (
	"git.urbach.dev/cli/q/src/errors"
	"git.urbach.dev/cli/q/src/eval"
	"git.urbach.dev/cli/q/src/scope"
	"git.urbach.dev/cli/q/src/token"
	"git.urbach.dev/cli/q/src/types"
	"git.urbach.dev/cli/q/src/x86"
)

// ResolveTypes parses the input and output types.
func (f *Function) ResolveTypes() error {
	for i, param := range f.Input {
		param.name = param.tokens[0].Text(f.File.Bytes)
		typeName := param.tokens[1:].Text(f.File.Bytes)
		param.typ = types.ByName(typeName, f.Package, f.All.Structs)

		if param.typ == nil {
			return errors.New(&errors.UnknownType{Name: typeName}, f.File, param.tokens[1].Position)
		}

		if f.IsExtern() {
			continue
		}

		uses := token.Count(f.Body, f.File.Bytes, token.Identifier, param.name)

		if uses == 0 && param.name != "_" {
			return errors.New(&errors.UnusedVariable{Name: param.name}, f.File, param.tokens[0].Position)
		}

		f.AddVariable(&scope.Variable{
			Name: param.name,
			Value: eval.Value{
				Kind:     eval.Register,
				Type:     param.typ,
				Register: x86.InputRegisters[i],
				Alive:    uses,
			},
		})
	}

	for _, param := range f.Output {
		typeName := param.tokens.Text(f.File.Bytes)
		param.typ = types.ByName(typeName, f.Package, f.All.Structs)

		if param.typ == nil {
			return errors.New(&errors.UnknownType{Name: typeName}, f.File, param.tokens[0].Position)
		}

		f.OutputTypes = append(f.OutputTypes, param.typ)
	}

	return nil
}
